using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using Syncfusion.Windows.Forms.Chart;

namespace POC.Chart
{
    public class LatencyChartController
    {
        private LatencyChart _latencyChart = null;

        public void Init(Dictionary<int, double> seqNumLatencyMap) 
        {
            _latencyChart = new LatencyChart();
            _latencyChart.Init();

            InitializeChart(seqNumLatencyMap);
            InitializeControlSettings();
        }

        public void ShowChart()
        {
            _latencyChart.ShowDialog();
        }

        protected void InitializeChart(Dictionary<int, double> seqNumLatencyMap)
        {
            ChartSeries series = _latencyChart.chart.Model.NewSeries();
            int min = 0;
            int max = 0;
            foreach (int seqNum in seqNumLatencyMap.Keys)
            {
                if (min == 0)
                    min = max = seqNum;
                if (min > seqNum)
                    min = seqNum;
                if (max < seqNum)
                    max = seqNum;

                series.Points.Add(seqNum, seqNumLatencyMap[seqNum]);
            }

            series.Type = ChartSeriesType.Scatter;
            series.ScatterConnectType = ScatterConnectType.Line;
            series.Style.DisplayShadow = false;
            series.Style.Symbol.Size = new Size(5, 5);
            series.Style.Symbol.Shape = ChartSymbolShape.Circle;
            series.PointsToolTipFormat = "{4}";
            series.PrepareStyle += new ChartPrepareStyleInfoHandler(OnPrepareStyle);
            _latencyChart.chart.Series.Add(series);

            _latencyChart.chart.Indexed = false;
            _latencyChart.chart.ShowToolTips = true;

            _latencyChart.chart.PrimaryXAxis.RangeType = ChartAxisRangeType.Set;
            _latencyChart.chart.PrimaryXAxis.Range = new MinMaxInfo(min - 1, max + 1, 1);
            _latencyChart.chart.PrimaryXAxis.DrawGrid = false;
            _latencyChart.chart.PrimaryXAxis.Title = "Inbound Message";
            _latencyChart.chart.PrimaryXAxis.LabelIntersectAction = ChartLabelIntersectAction.Rotate;
            _latencyChart.chart.ChartFormatAxisLabel += new ChartFormatAxisLabelEventHandler(OnChartFormatAxisLabel);
            
            _latencyChart.chart.PrimaryYAxis.DrawGrid = false;
            _latencyChart.chart.PrimaryYAxis.Title = "Latency (ms)";

            _latencyChart.chart.Text = "Latency of Inbound Messages";

            // Hide the legend
            _latencyChart.chart.Legend.Visible = false;
        }

        void OnPrepareStyle(object sender, ChartPrepareStyleInfoEventArgs args)
        {
            args.Style.ToolTip = _latencyChart.chart.Series[0].Points[args.Index].YValues[0].ToString();
            args.Handled = true;
        }

        void OnChartFormatAxisLabel(object sender, ChartFormatAxisLabelEventArgs e)
        {
            if (e.AxisOrientation == ChartOrientation.Horizontal)
            {
                e.Label = string.Empty;
                e.Handled = true;
            }
        }

        private void InitializeControlSettings()
        {
            _latencyChart.chart.Font = new System.Drawing.Font("Verdana", 10F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((System.Byte)(0)));

            // Specify a ChartPalette:
            Color[] color = new Color[] { Color.FromArgb(139, 47, 26)};
            _latencyChart.chart.Palette = ChartColorPalette.Custom;
            _latencyChart.chart.Model.ColorModel.CustomColors = color;

            _latencyChart.chart.LegendPosition = ChartDock.Top;
            _latencyChart.chart.HidePartialLabels = true;
        }
    }
}
